@@ -33,13 +33,14 @@
 #include "xfce-notify-window.h"
 #include "xfce-notify-enum-types.h"
 
-#define DEFAULT_EXPIRE_TIMEOUT    10000
-#define DEFAULT_NORMAL_OPACITY        0.85
-#define FADE_TIME                   800
-#define FADE_CHANGE_TIMEOUT          50
-#define DEFAULT_RADIUS               10.0
-#define DEFAULT_BORDER_WIDTH          2.0
-#define BORDER                        6
+#define DEFAULT_EXPIRE_TIMEOUT 10000
+#define DEFAULT_NORMAL_OPACITY 0.85
+#define FADE_TIME              800
+#define FADE_CHANGE_TIMEOUT    50
+#define DEFAULT_RADIUS         10.0
+#define DEFAULT_BORDER_WIDTH   2.0
+#define DEFAULT_PADDING        14.0
+#define BORDER                 6
 
 struct _XfceNotifyWindow
 {
@@ -53,8 +54,6 @@ struct _XfceNotifyWindow
 
     gboolean mouse_hover;
     cairo_path_t *bg_path;
-    cairo_path_t *close_btn_path;
-    GdkRegion *close_btn_region;
 
     gdouble normal_opacity;
 
@@ -108,9 +107,6 @@ static gboolean xfce_notify_window_button_release(GtkWidget *widget,
                                                   GdkEventButton *evt);
 static gboolean xfce_notify_window_configure_event(GtkWidget *widget,
                                                    GdkEventConfigure *evt);
-static void xfce_notify_window_style_set(GtkWidget *widget,
-                                         GtkStyle *previous_style);
-
 static gboolean xfce_notify_window_expire_timeout(gpointer data);
 static gboolean xfce_notify_window_fade_timeout(gpointer data);
 
@@ -138,7 +134,6 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
     widget_class->leave_notify_event = xfce_notify_window_enter_leave;
     widget_class->button_release_event = xfce_notify_window_button_release;
     widget_class->configure_event = xfce_notify_window_configure_event;
-    widget_class->style_set = xfce_notify_window_style_set;
 
     signals[SIG_CLOSED] = g_signal_new("closed",
                                        XFCE_TYPE_NOTIFY_WINDOW,
@@ -167,6 +162,13 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
                                                                GDK_TYPE_COLOR,
                                                                G_PARAM_READABLE));
 
+    gtk_widget_class_install_style_property(widget_class,
+                                            g_param_spec_boxed("border-color-hover",
+                                                               "border color hover",
+                                                               "the color of the border when hovering the notification",
+                                                               GDK_TYPE_COLOR,
+                                                               G_PARAM_READABLE));
+
     gtk_widget_class_install_style_property(widget_class,
                                             g_param_spec_double("border-radius",
                                                                 "border radius",
@@ -182,11 +184,19 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
                                                                 DEFAULT_BORDER_WIDTH,
                                                                 G_PARAM_READABLE));
     gtk_widget_class_install_style_property(widget_class,
-                                            g_param_spec_boolean("summary-bold",
-                                                                 "summary bold",
-                                                                 "whether or not to display the notification summary field in bold text",
-                                                                 FALSE,
-                                                                 G_PARAM_READABLE));
+                                            g_param_spec_double("border-width-hover",
+                                                                "border width hover",
+                                                                "the width of the border when hovering the notification",
+                                                                0.0, 8.0,
+                                                                DEFAULT_BORDER_WIDTH,
+                                                                G_PARAM_READABLE));
+    gtk_widget_class_install_style_property(widget_class,
+                                            g_param_spec_double("padding",
+                                                                "padding",
+                                                                "the padding of the text/icon to the notification's border",
+                                                                0.0, 30.0,
+                                                                DEFAULT_PADDING,
+                                                                G_PARAM_READABLE));
 }
 
 static void
@@ -194,13 +204,10 @@ xfce_notify_window_init(XfceNotifyWindow *window)
 {
     GdkScreen *screen;
     GtkWidget *tophbox, *align, *vbox;
-    gdouble border_radius = DEFAULT_RADIUS;
+    gdouble padding = DEFAULT_PADDING;
 
-    GTK_WINDOW(window)->type = GTK_WINDOW_TOPLEVEL;
     window->expire_timeout = DEFAULT_EXPIRE_TIMEOUT;
     window->normal_opacity = DEFAULT_NORMAL_OPACITY;
-    /* The summary widget needs to be initialized before style_set is called. gtk_widget_ensure_style calls style_set */
-    window->summary = gtk_label_new(NULL);
 
     gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
     gtk_window_stick(GTK_WINDOW(window));
@@ -226,11 +233,11 @@ xfce_notify_window_init(XfceNotifyWindow *window)
 
     gtk_widget_ensure_style(GTK_WIDGET(window));
     gtk_widget_style_get(GTK_WIDGET(window),
-                         "border-radius", &border_radius,
+                         "padding", &padding,
                          NULL);
 
     tophbox = gtk_hbox_new(FALSE, BORDER);
-    gtk_container_set_border_width(GTK_CONTAINER(tophbox), border_radius + 4);
+    gtk_container_set_border_width(GTK_CONTAINER(tophbox), padding);
     gtk_widget_show(tophbox);
     gtk_container_add(GTK_CONTAINER(window), tophbox);
 
@@ -247,11 +254,14 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gtk_widget_show(vbox);
     gtk_box_pack_start(GTK_BOX(tophbox), vbox, TRUE, TRUE, 0);
 
+    window->summary = gtk_label_new(NULL);
+    gtk_widget_set_name (window->summary, "summary");
     gtk_label_set_line_wrap(GTK_LABEL(window->summary), TRUE);
     gtk_misc_set_alignment(GTK_MISC(window->summary), 0.0, 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), window->summary, FALSE, FALSE, 0);
 
     window->body = gtk_label_new(NULL);
+    gtk_widget_set_name (window->body, "body");
     gtk_label_set_line_wrap(GTK_LABEL(window->body), TRUE);
     gtk_misc_set_alignment(GTK_MISC(window->body), 0.0, 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), window->body, TRUE, TRUE, 0);
@@ -269,6 +279,7 @@ xfce_notify_window_start_expiration(XfceNotifyWindow *window)
 {
     if(window->expire_timeout) {
         GTimeVal ct;
+        guint timeout;
         gboolean fade_transparent;
 
         g_get_current_time(&ct);
@@ -276,10 +287,15 @@ xfce_notify_window_start_expiration(XfceNotifyWindow *window)
         fade_transparent =
             gdk_screen_is_composited(gtk_window_get_screen(GTK_WINDOW (window)));
 
+        if(!fade_transparent)
+            timeout = window->expire_timeout;
+        else if(window->expire_timeout > FADE_TIME)
+            timeout = window->expire_timeout - FADE_TIME;
+        else
+            timeout = FADE_TIME;
+
         window->expire_start_timestamp = ct.tv_sec * 1000 + ct.tv_usec / 1000;
-        window->expire_id = g_timeout_add(fade_transparent
-                                          ? window->expire_timeout - FADE_TIME
-                                          : window->expire_timeout,
+        window->expire_id = g_timeout_add(timeout,
                                           xfce_notify_window_expire_timeout,
                                           window);
     }
@@ -326,16 +342,6 @@ xfce_notify_window_unrealize(GtkWidget *widget)
         cairo_path_destroy(window->bg_path);
         window->bg_path = NULL;
     }
-
-    if(window->close_btn_path) {
-        cairo_path_destroy(window->close_btn_path);
-        window->close_btn_path = NULL;
-    }
-
-    if(window->close_btn_region) {
-        gdk_region_destroy(window->close_btn_region);
-        window->close_btn_region = NULL;
-    }
 }
 
 static inline GdkRegion *
@@ -403,6 +409,12 @@ xfce_notify_window_ensure_bg_path(XfceNotifyWindow *window,
 {
     GtkWidget *widget = GTK_WIDGET(window);
     gdouble radius = DEFAULT_RADIUS;
+    gdouble border_width = DEFAULT_BORDER_WIDTH;
+
+    /* this secifies the border_padding from the edges in order to make
+     * sure the border completely fits into the drawing area */
+    gdouble border_padding = 0.0;
+
     cairo_path_t *flat_path;
     GdkRegion *region;
     GdkFillRule fill_rule;
@@ -413,27 +425,45 @@ xfce_notify_window_ensure_bg_path(XfceNotifyWindow *window,
 
     gtk_widget_size_request(GTK_WIDGET(window), &req);
 
-    gtk_widget_style_get(widget,
-                         "border-radius", &radius,
-                         NULL);
+    if (!window->mouse_hover) {
+        gtk_widget_style_get(widget,
+                             "border-radius", &radius,
+                             "border-width", &border_width,
+                             NULL);
+    } else {
+        gtk_widget_style_get(widget,
+                             "border-radius", &radius,
+                             "border-width-hover", &border_width,
+                             NULL);
+    }
+
+    border_padding = border_width / 2.0;
 
     if(radius < 0.1) {
         cairo_rectangle(cr, 0, 0, widget->allocation.width,
                         widget->allocation.height);
     } else {
-        cairo_move_to(cr, 0, radius);
-        cairo_arc(cr, radius, radius, radius, M_PI, 3.0*M_PI/2.0);
-        cairo_line_to(cr, widget->allocation.width - radius, 0);
-        cairo_arc(cr, widget->allocation.width - radius, radius, radius,
+        cairo_move_to(cr, border_padding, radius + border_padding);
+        cairo_arc(cr, radius + border_padding,
+                  radius + border_padding, radius,
+                  M_PI, 3.0*M_PI/2.0);
+        cairo_line_to(cr,
+                      widget->allocation.width - radius - border_padding,
+                      border_padding);
+        cairo_arc(cr,
+                  widget->allocation.width - radius - border_padding,
+                  radius + border_padding, radius,
                   3.0*M_PI/2.0, 0.0);
-        cairo_line_to(cr, widget->allocation.width,
-                      widget->allocation.height - radius);
-        cairo_arc(cr, widget->allocation.width - radius,
-                  widget->allocation.height - radius, radius,
-                  0.0, M_PI/2.0);
-        cairo_line_to(cr, radius, widget->allocation.height);
-        cairo_arc(cr, radius, widget->allocation.height - radius, radius,
-                  M_PI/2.0, M_PI);
+        cairo_line_to(cr, widget->allocation.width - border_padding,
+                      widget->allocation.height - radius - border_padding);
+        cairo_arc(cr, widget->allocation.width - radius - border_padding,
+                  widget->allocation.height - radius - border_padding,
+                  radius, 0.0, M_PI/2.0);
+        cairo_line_to(cr, radius + border_padding,
+                      widget->allocation.height - border_padding);
+        cairo_arc(cr, radius + border_padding,
+                  widget->allocation.height - radius - border_padding,
+                  radius, M_PI/2.0, M_PI);
         cairo_close_path(cr);
     }
 
@@ -468,6 +498,8 @@ xfce_notify_window_expose(GtkWidget *widget,
     cairo_t *cr;
     GList *children, *l;
     cairo_path_t *bg_path;
+    GdkColor *border_color = NULL;
+    gdouble border_width = DEFAULT_BORDER_WIDTH;
 
     if(evt->count != 0)
         return FALSE;
@@ -502,52 +534,26 @@ xfce_notify_window_expose(GtkWidget *widget,
         }
     }
 
-
     if(window->mouse_hover) {
-        GdkColor *border_color = NULL;
-        gdouble border_width = DEFAULT_BORDER_WIDTH;
-
+        gtk_widget_style_get(widget,
+                             "border-color-hover", &border_color,
+                             "border-width-hover", &border_width,
+                             NULL);
+    } else {
         gtk_widget_style_get(widget,
                              "border-color", &border_color,
                              "border-width", &border_width,
                              NULL);
-
-        cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-        if(border_color)
-            gdk_cairo_set_source_color(cr, border_color);
-        else
-            cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
-        cairo_set_line_width(cr, border_width);
-
-        cairo_stroke(cr);
-
-        /* draw a circle with an X in it */
-        if(!window->close_btn_path) {
-            cairo_path_t *flat_path;
-            GdkFillRule fill_rule;
-
-            cairo_arc(cr, widget->allocation.width - 12., 12., 7.5, 0., 2*M_PI);
-            window->close_btn_path = cairo_copy_path(cr);
-
-            flat_path = cairo_copy_path_flat(cr);
-            fill_rule = (cairo_get_fill_rule(cr) == CAIRO_FILL_RULE_WINDING
-                         ? GDK_WINDING_RULE : GDK_EVEN_ODD_RULE);
-            window->close_btn_region = xfce_gdk_region_from_cairo_flat_path(flat_path,
-                                                                            fill_rule);
-            cairo_path_destroy(flat_path);
-        } else
-            cairo_append_path(cr, window->close_btn_path);
-        cairo_set_line_width(cr, 1.5);
-        cairo_stroke(cr);
-
-        cairo_move_to(cr, widget->allocation.width - 8., 8.);
-        cairo_line_to(cr, widget->allocation.width - 16., 16.);
-        cairo_stroke(cr);
-        cairo_move_to(cr, widget->allocation.width - 16., 8.);
-        cairo_line_to(cr, widget->allocation.width - 8., 16.);
-        cairo_stroke(cr);
     }
 
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    if(border_color)
+        gdk_cairo_set_source_color(cr, border_color);
+    else
+        cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+    cairo_set_line_width(cr, border_width);
+
+    cairo_stroke(cr);
     cairo_destroy(cr);
 
     children = gtk_container_get_children(GTK_CONTAINER(widget));
@@ -593,15 +599,6 @@ static gboolean
 xfce_notify_window_button_release(GtkWidget *widget,
                                   GdkEventButton *evt)
 {
-    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
-
-    if(!window->close_btn_region
-       || !gdk_region_point_in(window->close_btn_region, evt->x, evt->y))
-    {
-        g_signal_emit(G_OBJECT(widget), signals[SIG_ACTION_INVOKED], 0,
-                      "default");
-    }
-
     g_signal_emit(G_OBJECT(widget), signals[SIG_CLOSED], 0,
                   XFCE_NOTIFY_CLOSE_REASON_DISMISSED);
 
@@ -623,42 +620,11 @@ xfce_notify_window_configure_event(GtkWidget *widget,
         window->bg_path = NULL;
     }
 
-    if(window->close_btn_path) {
-        cairo_path_destroy(window->close_btn_path);
-        window->close_btn_path = NULL;
-    }
-
-    if(window->close_btn_region) {
-        gdk_region_destroy(window->close_btn_region);
-        window->close_btn_region = NULL;
-    }
-
     gtk_widget_queue_draw(widget);
 
     return ret;
 }
 
-static void
-xfce_notify_window_style_set(GtkWidget *widget,
-                             GtkStyle *previous_style)
-{
-    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
-    gboolean summary_bold = FALSE;
-    GtkStyle *style;
-    PangoFontDescription *pfd;
-
-    gtk_widget_style_get(widget,
-                         "summary-bold", &summary_bold,
-                         NULL);
-    if(summary_bold) {
-        style = gtk_widget_get_style(window->summary);
-        pfd = pango_font_description_copy(style->font_desc);
-        pango_font_description_set_weight(pfd, PANGO_WEIGHT_BOLD);
-        gtk_widget_modify_font(window->summary, pfd);
-        pango_font_description_free(pfd);
-    }
-}
-
 
 
 static gboolean
@@ -719,7 +685,7 @@ xfce_notify_window_button_clicked(GtkWidget *widget,
 
     g_signal_emit(G_OBJECT(window), signals[SIG_ACTION_INVOKED], 0,
                   action_id);
-    g_signal_emit(G_OBJECT(widget), signals[SIG_CLOSED], 0,
+    g_signal_emit(G_OBJECT(window), signals[SIG_CLOSED], 0,
                   XFCE_NOTIFY_CLOSE_REASON_DISMISSED);
 }
 
@@ -746,168 +712,6 @@ elem_to_string(gconstpointer elem_p)
     }
 }
 
-static gchar *
-xfce_notify_window_validate_escape_markup(const gchar *str)
-{
-    GString *gstr;
-    const gchar *p;
-    GQueue *open_elems;
-    gconstpointer tmp;
-
-    if(!str)
-        return NULL;
-
-    open_elems = g_queue_new();
-    gstr = g_string_sized_new(strlen(str));
-    p = str;
-
-    while(*p) {
-        if('<' == *p) {
-            if('b' == *(p+1) && '>' == *(p+2)) {
-                g_queue_push_head(open_elems, ELEM_B);
-                g_string_append(gstr, "<b>");
-                p += 3;
-            } else if('i' == *(p+1) && '>' == *(p+2)) {
-                g_queue_push_head(open_elems, ELEM_I);
-                g_string_append(gstr, "<i>");
-                p += 3;
-            } else if('u' == *(p+1) && '>' == *(p+2)) {
-                g_queue_push_head(open_elems, ELEM_U);
-                g_string_append(gstr, "<u>");
-                p += 3;
-            } else if('a' == *(p+1) && ' ' == *(p+2)) {
-                gchar *aend;
-
-                g_queue_push_head(open_elems, ELEM_A);
-
-                aend = strchr(p+3, '>');
-                if(!aend) {
-                    g_warning("Bad markup in <a>: %s", str);
-                    goto out_err;
-                }
-#if GTK_CHECK_VERSION(2,16,0)
-                /* only support links with gtk 2.16 */
-                g_string_append_len(gstr, p, aend - p + 1);
-#endif
-                p = aend + 1;
-            } else if(!strncmp(p+1, "img ", 4)) {
-                /* don't currently support images; extract alt text
-                 * if available */
-                gchar *imgend, *altbegin, *altend = NULL;
-
-                altbegin = strstr(p+5, "alt=\"");
-                if(altbegin) {
-                    altbegin += 5;
-                    altend = strchr(altbegin, '"');
-                    if(!altend) {
-                        g_warning("End of <img> alt text not found");
-                        goto out_err;
-                    }
-                }
-
-                imgend = strstr(altend ? altend+1 : p+4, "/>");
-                if(!imgend) {
-                    g_warning("Unclosed <img> tag");
-                    goto out_err;
-                }
-
-                if(altbegin) {
-                    /* put the alt text in the label */
-                    g_string_append_c(gstr, '[');
-                    g_string_append(gstr, _("image: "));
-                    p = altbegin;
-                    while(p < altend) {
-                        if(*p == '<')
-                            g_string_append(gstr, "&lt;");
-                        else if(*p == '>')
-                            g_string_append(gstr, "&gt;");
-                        else if(*p == '&')
-                            g_string_append(gstr, "&amp;");
-                        else
-                            g_string_append_c(gstr, *p);
-                        p++;
-                    }
-                    g_string_append_c(gstr, ']');
-                }
-
-                p = imgend + 2;
-            } else if('/' == *(p+1)) {
-                if('b' == *(p+2) && '>' == *(p+3)) {
-                    tmp = g_queue_pop_head(open_elems);
-                    if(tmp != ELEM_B) {
-                        g_warning("Bad markup: closing <b> when %s expected",
-                                  elem_to_string(tmp));
-                        goto out_err;
-                    }
-                    g_string_append(gstr, "</b>");
-                    p += 4;
-                } else if('i' == *(p+2) && '>' == *(p+3)) {
-                    tmp = g_queue_pop_head(open_elems);
-                    if(tmp != ELEM_I) {
-                        g_warning("Bad markup: closing <i> when %s expected",
-                                  elem_to_string(tmp));
-                        goto out_err;
-                    }
-                    g_string_append(gstr, "</i>");
-                    p += 4;
-                } else if('u' == *(p+2) && '>' == *(p+3)) {
-                    tmp = g_queue_pop_head(open_elems);
-                    if(tmp != ELEM_U) {
-                        g_warning("Bad markup: closing <u> when %s expected",
-                                  elem_to_string(tmp));
-                        goto out_err;
-                    }
-                    g_string_append(gstr, "</u>");
-                    p += 4;
-                } else if('a' == *(p+2) && '>' == *(p+3)) {
-                    tmp = g_queue_pop_head(open_elems);
-                    if(tmp != ELEM_A) {
-                        g_warning("Bad markup: closing <a> when %s expected",
-                                  elem_to_string(tmp));
-                        goto out_err;
-                    }
-#if GTK_CHECK_VERSION(2,16,0)
-                    g_string_append(gstr, "</a>");
-#endif
-                    p += 4;
-                } else {
-                    g_string_append(gstr, "&gt;");
-                    p++;
-                }
-            } else {
-                g_string_append(gstr, "&lt;");
-                p++;
-            }
-        } else if('>' == *p) {
-            g_string_append(gstr, "&gt;");
-            p++;
-        } else {
-            const gchar *next = g_utf8_next_char(p);
-
-            if(!next) {
-                g_critical("Bad UTF-8 in string");
-                goto out_err;
-            }
-
-            g_string_append_len(gstr, p, next - p);
-            p = next;
-        }
-    }
-
-    p = gstr->str;
-    g_string_free(gstr, FALSE);
-    g_queue_free(open_elems);
-
-    return (gchar *)p;
-
-out_err:
-    g_string_free(gstr, TRUE);
-    g_queue_free(open_elems);
-    return NULL;
-}
-
-
-
 GtkWidget *
 xfce_notify_window_new(void)
 {
@@ -935,7 +739,8 @@ xfce_notify_window_new_with_actions(const gchar *summary,
 {
     XfceNotifyWindow *window;
 
-    window = g_object_new(XFCE_TYPE_NOTIFY_WINDOW, NULL);
+    window = g_object_new(XFCE_TYPE_NOTIFY_WINDOW,
+                          "type", GTK_WINDOW_TOPLEVEL, NULL);
 
     xfce_notify_window_set_summary(window, summary);
     xfce_notify_window_set_body(window, body);
@@ -975,14 +780,10 @@ xfce_notify_window_set_body(XfceNotifyWindow *window,
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
     if(body && *body) {
-        gchar *markup = xfce_notify_window_validate_escape_markup(body);
-        if(!markup)
-            return;
-
-        gtk_label_set_markup(GTK_LABEL(window->body), markup);
+        gtk_label_set_markup(GTK_LABEL(window->body), body);
 
         gtk_widget_show(window->body);
-        g_free(markup);
+
         window->has_body_text = TRUE;
     } else {
         gtk_label_set_markup(GTK_LABEL(window->body), "");
@@ -1041,6 +842,7 @@ xfce_notify_window_set_icon_name(XfceNotifyWindow *window,
                                  const gchar *icon_name)
 {
     gboolean icon_set = FALSE;
+    gchar *filename;
 
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
@@ -1052,6 +854,12 @@ xfce_notify_window_set_icon_name(XfceNotifyWindow *window,
 
         if(g_path_is_absolute(icon_name))
           pix = gdk_pixbuf_new_from_file_at_size(icon_name, w, h, NULL);
+        else if(g_str_has_prefix (icon_name, "file://")) {
+            filename = g_filename_from_uri(icon_name, NULL, NULL);
+            if(filename)
+              pix = gdk_pixbuf_new_from_file_at_size(filename, w, h, NULL);
+            g_free(filename);
+          }
         else
           pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
                                        icon_name,
